// Copyright (c) 1999 Brian Wellington (bwelling@xbill.org)
// Portions Copyright (c) 1999 Network Associates, Inc.

package org.xbill.DNS;

import java.io.IOException;
import java.net.InetAddress;

import org.xbill.DNS.utils.DataByteInputStream;
import org.xbill.DNS.utils.DataByteOutputStream;
import org.xbill.DNS.utils.MyStringTokenizer;

/**
 * Address Record - maps a domain name to an Internet address
 * 
 * @author Brian Wellington
 */

public class ARecord extends Record {

	private InetAddress address;

	/**
	 * Creates an A Record from the given data
	 * 
	 * @param address The address that the name refers to
	 */
	public ARecord(Name _name, short _dclass, int _ttl, InetAddress _address) throws IOException {
		super(_name, Type.A, _dclass, _ttl);
		address = _address;
	}

	ARecord() {
	}

	ARecord(Name _name, short _dclass, int _ttl, int length, DataByteInputStream in, Compression c) throws IOException {
		super(_name, Type.A, _dclass, _ttl);

		if (in == null) return;

		final byte[] data = new byte[4];
		in.read(data);

		String s = new String();
		s = (data[0] & 0xFF) + "." + (data[1] & 0xFF) + "." + (data[2] & 0xFF) + "." + (data[3] & 0xFF);
		address = InetAddress.getByName(s);
	}

	ARecord(Name _name, short _dclass, int _ttl, MyStringTokenizer st, Name origin) throws IOException {
		super(_name, Type.A, _dclass, _ttl);
		final String s = st.nextToken();
		if (s.equals("@me@")) {
			address = InetAddress.getLocalHost();
			if (address.equals(InetAddress.getByName("127.0.0.1"))) {
				throw new IOException("InetAddress.getLocalHost() is broken.");
			}
		} else {
			address = InetAddress.getByName(s);
		}
	}

	/** Returns the Internet address */
	public InetAddress getAddress() {
		return address;
	}

	/** Converts to a String */
	@Override
	public String toString() {
		final StringBuffer sb = toStringNoData();
		if (address != null) {
			sb.append(address.getHostAddress());
		}
		return sb.toString();
	}

	@Override
	void rrToWire(DataByteOutputStream out, Compression c) throws IOException {
		if (address == null) return;

		out.write(address.getAddress());
	}

}
