// Copyright (c) 1999 Brian Wellington (bwelling@xbill.org)
// Portions Copyright (c) 1999 Network Associates, Inc.

package org.xbill.DNS;

import java.io.IOException;

import org.xbill.DNS.utils.DataByteInputStream;
import org.xbill.DNS.utils.DataByteOutputStream;
import org.xbill.DNS.utils.MyStringTokenizer;

/**
 * Implements MX and KX records, which have identical formats
 * 
 * @author Brian Wellington
 */

public class MX_KXRecord extends Record {

	private short priority;
	private Name target;

	public MX_KXRecord(Name _name, short _type, short _dclass, int _ttl, int _priority, Name _target) {
		super(_name, _type, _dclass, _ttl);
		priority = (short) _priority;
		target = _target;
	}

	protected MX_KXRecord() {
	}

	protected MX_KXRecord(Name _name, short _type, short _dclass, int _ttl, int length, DataByteInputStream in,
			Compression c) throws IOException {
		super(_name, _type, _dclass, _ttl);
		if (in == null) return;
		priority = (short) in.readUnsignedShort();
		target = new Name(in, c);
	}

	protected MX_KXRecord(Name _name, short _type, short _dclass, int _ttl, MyStringTokenizer st, Name origin)
			throws IOException {
		super(_name, _type, _dclass, _ttl);
		priority = Short.parseShort(st.nextToken());
		target = new Name(st.nextToken(), origin);
	}

	/** Returns the priority of this record */
	public short getPriority() {
		return priority;
	}

	/** Returns the target of the record */
	public Name getTarget() {
		return target;
	}

	/** Converts to a String */
	@Override
	public String toString() {
		final StringBuffer sb = toStringNoData();
		if (target != null) {
			sb.append(priority);
			sb.append(" ");
			sb.append(target);
		}
		return sb.toString();
	}

	@Override
	void rrToWire(DataByteOutputStream out, Compression c) throws IOException {
		if (target == null) return;

		out.writeShort(priority);
		target.toWire(out, null);
	}

	@Override
	void rrToWireCanonical(DataByteOutputStream out) throws IOException {
		if (target == null) return;

		out.writeShort(priority);
		target.toWireCanonical(out);
	}

}
