// Copyright (c) 1999 Brian Wellington (bwelling@xbill.org)
// Portions Copyright (c) 1999 Network Associates, Inc.

package org.xbill.DNS;

import java.io.IOException;

import org.xbill.DNS.utils.DataByteInputStream;
import org.xbill.DNS.utils.DataByteOutputStream;
import org.xbill.DNS.utils.MyStringTokenizer;

/**
 * Implements NS, CNAME, and PTR records, which have identical formats
 * 
 * @author Brian Wellington
 */

public class NS_CNAME_PTRRecord extends Record {

	private Name target;

	public NS_CNAME_PTRRecord(Name _name, short _type, short _dclass, int _ttl, Name _target) {
		super(_name, _type, _dclass, _ttl);
		target = _target;
	}

	protected NS_CNAME_PTRRecord() {
	}

	protected NS_CNAME_PTRRecord(Name _name, short _type, short _dclass, int _ttl, int length, DataByteInputStream in,
			Compression c) throws IOException {
		super(_name, _type, _dclass, _ttl);
		if (in == null) return;
		target = new Name(in, c);
	}

	protected NS_CNAME_PTRRecord(Name _name, short _type, short _dclass, int _ttl, MyStringTokenizer st, Name origin)
			throws IOException {
		super(_name, _type, _dclass, _ttl);
		target = new Name(st.nextToken(), origin);
	}

	/** Gets the target of the NS, CNAME, or PTR Record */
	public Name getTarget() {
		return target;
	}

	/** Converts the NS, CNAME, or PTR Record to a String */
	@Override
	public String toString() {
		final StringBuffer sb = toStringNoData();
		if (target != null) {
			sb.append(target);
		}
		return sb.toString();
	}

	@Override
	void rrToWire(DataByteOutputStream out, Compression c) throws IOException {
		if (target == null) return;

		target.toWire(out, c);
	}

	@Override
	void rrToWireCanonical(DataByteOutputStream out) throws IOException {
		if (target == null) return;

		target.toWireCanonical(out);
	}

}
