// Copyright (c) 1999 Brian Wellington (bwelling@xbill.org)
// Portions Copyright (c) 1999 Network Associates, Inc.

package org.xbill.DNS;

import java.io.IOException;

/**
 * Interface describing a resolver.
 * 
 * @author Brian Wellington
 */

public interface Resolver {

	/**
	 * Sends a message, and waits for a response
	 * 
	 * @return The response
	 */
	public Message send(Message query) throws IOException;

	/**
	 * Sends a zone transfer message, and waits for a response
	 * 
	 * @return The response
	 */
	public Message sendAXFR(Message query) throws IOException;

	/** Sets the EDNS version used on outgoing messages (only 0 is meaningful) */
	public void setEDNS(int level);

	/** Sets whether truncated responses will be returned */
	public void setIgnoreTruncation(boolean flag);

	/** Sets the port to communicate with on the server */
	public void setPort(int port);

	/** Sets whether TCP connections will be sent by default */
	public void setTCP(boolean flag);

	/** Sets the amount of time to wait for a response before giving up */
	public void setTimeout(int secs);

	/**
	 * Specifies the TSIG key (with the same name as the local host) that
	 * messages will be signed with
	 */
	public void setTSIGKey(String key);

	/** Specifies the TSIG key that messages will be signed with */
	public void setTSIGKey(String name, String key);

}
