// Copyright (c) 1999 Brian Wellington (bwelling@xbill.org)
// Portions Copyright (c) 1999 Network Associates, Inc.

package org.xbill.DNS;

import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;

import org.xbill.DNS.utils.DataByteInputStream;
import org.xbill.DNS.utils.DataByteOutputStream;
import org.xbill.DNS.utils.MyStringTokenizer;

/**
 * Text - stores text strings
 * 
 * @author Brian Wellington
 */

public class TXTRecord extends Record {

	private Vector<String> strings;

	/**
	 * Creates a TXT Record from the given data
	 * 
	 * @param strings One text string
	 */
	public TXTRecord(Name _name, short _dclass, int _ttl, String _string) throws IOException {
		super(_name, Type.TXT, _dclass, _ttl);
		strings = new Vector<String>();
		strings.addElement(_string);
	}

	/**
	 * Creates a TXT Record from the given data
	 * 
	 * @param strings The text strings
	 */
	public TXTRecord(Name _name, short _dclass, int _ttl, Vector<String> _strings) throws IOException {
		super(_name, Type.TXT, _dclass, _ttl);
		strings = _strings;
	}

	TXTRecord(Name _name, short _dclass, int _ttl, int length, DataByteInputStream in, Compression c)
			throws IOException {
		super(_name, Type.TXT, _dclass, _ttl);
		if (in == null) return;
		int count = 0;
		strings = new Vector<String>();
		while (count < length) {
			final int len = in.readByte();
			final byte[] b = new byte[len];
			in.read(b);
			count += len + 1;
			strings.addElement(new String(b));
		}
	}

	TXTRecord(Name _name, short _dclass, int _ttl, MyStringTokenizer st, Name origin) throws IOException {
		super(_name, Type.TXT, _dclass, _ttl);
		strings = new Vector<String>();
		while (st.hasMoreTokens()) {
			strings.addElement(st.nextToken());
		}
	}

	/** Returns the text strings */
	public Vector<String> getStrings() {
		return strings;
	}

	/** converts to a String */
	@Override
	public String toString() {
		final StringBuffer sb = toStringNoData();
		if (strings != null) {
			final Enumeration<String> e = strings.elements();
			while (e.hasMoreElements()) {
				final String s = e.nextElement();
				sb.append("\"");
				sb.append(s);
				sb.append("\" ");
			}
		}
		return sb.toString();
	}

	@Override
	void rrToWire(DataByteOutputStream out, Compression c) throws IOException {
		if (strings == null) return;

		final Enumeration<String> e = strings.elements();
		while (e.hasMoreElements()) {
			final String s = e.nextElement();
			out.writeString(s);
		}
	}

}
