// Copyright (c) 1999 Brian Wellington (bwelling@xbill.org)
// Portions Copyright (c) 1999 Network Associates, Inc.

package org.xbill.DNS;

import java.io.IOException;

import org.xbill.DNS.utils.DataByteInputStream;
import org.xbill.DNS.utils.DataByteOutputStream;
import org.xbill.DNS.utils.MyStringTokenizer;

/**
 * A class implementing Records of unknown and/or unimplemented types. This
 * class can only be initialized using static Record initializers.
 * 
 * @author Brian Wellington
 */

public class UNKRecord extends Record {

	private byte[] data;

	UNKRecord() {
	}

	UNKRecord(Name _name, short _type, short _dclass, int _ttl, int length, DataByteInputStream in, Compression c)
			throws IOException {
		super(_name, _type, _dclass, _ttl);
		if (in == null) return;
		if (length > 0) {
			data = new byte[length];
			in.read(data);
		} else {
			data = null;
		}
	}

	UNKRecord(Name _name, short _type, short _dclass, int _ttl, MyStringTokenizer st, Name origin) throws IOException {
		super(_name, _type, _dclass, _ttl);
		throw new IOException("Unknown type: " + type);
	}

	/** Converts this Record to the String "unknown format" */
	@Override
	public String toString() {
		final StringBuffer sb = toStringNoData();
		if (data != null) {
			sb.append("<unknown format>");
		}
		return sb.toString();
	}

	@Override
	void rrToWire(DataByteOutputStream out, Compression c) throws IOException {
		if (data != null) {
			out.write(data);
		}
	}

}
