// Copyright (c) 1999 Brian Wellington (bwelling@xbill.org)
// Portions Copyright (c) 1999 Network Associates, Inc.

package org.xbill.DNS.utils;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;

/**
 * Routines for converting between Strings of hex-encoded data and arrays of
 * binary data. This is not actually used by DNS.
 * 
 * @author Brian Wellington
 */

public class Base16 {

	static private final String Base16 = "0123456789ABCDEF";

	private Base16() {
	}

	/**
	 * Convert a hex-encoded String to binary data
	 * 
	 * @param b A String containing the encoded data
	 * @return An array containing the binary data, or null if the string is
	 *         invalid
	 */
	public static byte[] fromString(String str) {
		if (str.length() % 2 != 0) return null;
		final ByteArrayOutputStream bs = new ByteArrayOutputStream();
		final DataOutputStream ds = new DataOutputStream(bs);

		for (int i = 0; i < str.length(); i += 2) {
			final byte high = (byte) Base16.indexOf(Character.toUpperCase(str.charAt(i)));
			final byte low = (byte) Base16.indexOf(Character.toUpperCase(str.charAt(i + 1)));
			try {
				ds.writeByte((high << 4) + low);
			} catch (final IOException e) {
			}
		}
		return bs.toByteArray();
	}

	/**
	 * Convert binary data to a hex-encoded String
	 * 
	 * @param b An array containing binary data
	 * @return A String containing the encoded data
	 */
	public static String toString(byte[] b) {
		final ByteArrayOutputStream os = new ByteArrayOutputStream();

		for (final byte element : b) {
			final short value = (short) (element & 0xFF);
			final byte high = (byte) (value >> 4);
			final byte low = (byte) (value & 0xF);
			os.write(Base16.charAt(high));
			os.write(Base16.charAt(low));
		}
		return new String(os.toByteArray());
	}

}
